/*
 * Copyright (c) 1999 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software and design ideas developed by the Java 
 *    Apache Project (http://java.apache.org/)."
 *
 * 4. The names "Cocoon", "Cocoon Servlet" and "Java Apache Project" must 
 *    not be used to endorse or promote products derived from this software 
 *    without prior written permission.
 *
 * 5. Products derived from this software may not be called "Cocoon"
 *    nor may "Cocoon" and "Java Apache Project" appear in their names without 
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software and design ideas developed by the Java 
 *    Apache Project (http://java.apache.org/)."
 *           
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *           
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Project. For more information
 * on the Java Apache Project please see <http://java.apache.org/>.
 */

package org.apache.cocoon.cache;

/**
 * An individual object contained in a cache.
 *
 * @author <a href="mailto:ritter@wt.net">Greg Ritter</a>
 * @version $Revision: 1.1 $ $Date: 1999/03/24 13:46:05 $
 */
 
public class CachedObject {
	
    Object contents;
    long lastModified;
    long lastAccessed;

    /**
     * Create a new CachedObject with the specified timestamp.
     * @param contents the object to cache
     * @param timestamp the timestamp to be used as the last modified
     * and last accessed time for this CachedObject
     */
    public CachedObject(Object contents, long timestamp) {
        this.contents = contents;
        this.lastModified = timestamp;
        this.lastAccessed = timestamp;
    }

    /**
     * Create a new CachedObject.
     * Use the current system time as the timestamp.
     * @param object the object to cache
     */
    public CachedObject(Object contents) {
	    this(contents,System.currentTimeMillis());
    }

    /**
     * Get the last access time for this CachedObject.
     * @return the last access time
     */
    public long getLastAccessed() {
	    return lastAccessed;
    }

    /**
     * Get the last modification time for this CachedObject.
     * @return the last modification time
     */
    public long getLastModified() {
	    return lastModified;
    }

    /**
     * Get the contents of this CachedObject.
     * @return the contents of this CachedObject
     */
    public Object getContents() {
        lastAccessed = System.currentTimeMillis();
        return contents;
    }

    /**
     * Get a textual representation of this CachedObject.
     * @return a textual representation of this CachedObject
     */
    public String toString() {
    	return getContents().toString();
    }
}