/*
 * Copyright (c) 1999 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software and design ideas developed by the Java 
 *    Apache Project (http://java.apache.org/)."
 *
 * 4. The names "Cocoon", "Cocoon Servlet" and "Java Apache Project" must 
 *    not be used to endorse or promote products derived from this software 
 *    without prior written permission.
 *
 * 5. Products derived from this software may not be called "Cocoon"
 *    nor may "Cocoon" and "Java Apache Project" appear in their names without 
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software and design ideas developed by the Java 
 *    Apache Project (http://java.apache.org/)."
 *           
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *           
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Project. For more information
 * on the Java Apache Project please see <http://java.apache.org/>.
 */
 
package org.apache.cocoon.processor;

import java.io.*;
import java.net.*;
import java.util.*;
import org.w3c.dom.*;
import org.apache.cocoon.parser.*;
import org.apache.cocoon.framework.*;

/**
 * This class abstracts the XSL processor interface.
 *
 * @author <a href="mailto:stefano@apache.org">Stefano Mazzocchi</a>
 * @version $Revision: 1.2 $ $Date: 1999/03/22 23:50:08 $
 */

public abstract class AbstractXSLProcessor implements Processor, Configurable {
    
    public static final String STYLESHEET_PI_NAME = "xml-stylesheet";
    public static final String TYPE_ATTRIBUTE = "type";
    public static final String HREF_ATTRIBUTE = "href";
    public static final String MEDIA_ATTRIBUTE = "media";
	public static final String DEFAULT_BROWSER = "unknown";
	
    public static final String STYLESHEET_ERROR = "Could not associate stylesheet to document: ";
	public static final String STYLESHEET_NOT_FOUND = "No &lt;?" + STYLESHEET_PI_NAME + " ?&gt PI found.";
    public static final String MALFORMED_URL = " is a malformed URL.";

    public Parser parser;
    
    public void init(Configurations conf) {
        this.parser = (org.apache.cocoon.parser.Parser) conf.get("parser");
    }

	public URL getStylesheet(Document document, Hashtable parameters) throws ProcessorException {
        Hashtable links = getStylesheetsForBrowsers(document, (String) parameters.get("path"));
        String url = (String) links.get(parameters.get("browser"));

		try {
            if (url != null) {
                return new URL(url);
            } else {
            	url = (String) links.get(DEFAULT_BROWSER);
            	if (url != null) {
                    return new URL(url);
            	} else {
                    throw new ProcessorException(STYLESHEET_ERROR + STYLESHEET_NOT_FOUND);
            	}
            }
    	} catch (MalformedURLException e) {
    		throw new ProcessorException(STYLESHEET_ERROR + url + MALFORMED_URL);
    	}
    }
    
    public Hashtable getStylesheetsForBrowsers(Document document, String path) {
        Hashtable links = new Hashtable();
        NodeList nodelist = document.getChildNodes();
        int i = nodelist.getLength();
        for(int j = 0; j < i; j++) {
            Node node = nodelist.item(j);
            if(node.getNodeType() == 7) {
                ProcessingInstruction pi = (ProcessingInstruction) node;
                if(pi.getNodeName().equals(STYLESHEET_PI_NAME)) {
                    boolean validXSL = true;
                    
                    for(StringTokenizer st = new StringTokenizer(pi.getNodeValue(), " \t="); st.hasMoreTokens();) {
                        if(st.nextToken().equals(TYPE_ATTRIBUTE)) {
                            String token = st.nextToken();
                            token = token.substring(1, token.length() - 1);
                            if(!token.equals("text/xsl")) validXSL = false;
                        }
                    }

                    if(validXSL) {
                        String url = null;
                        String agent = null;
                        
                        for(StringTokenizer st = new StringTokenizer(pi.getNodeValue(), " \t="); st.hasMoreTokens();) {
                            if(st.nextToken().equals(HREF_ATTRIBUTE)) {
                                String href = st.nextToken();
                                href = href.substring(1, href.length() - 1);
                                if (href.indexOf("://") > 0) {
                                    url = href;
                                } else {
                                    url = "file:" + path + href;
                                }
                            }
                        }
                        
                        for(StringTokenizer st = new StringTokenizer(pi.getNodeValue(), " \t="); st.hasMoreTokens();) {
                            if(st.nextToken().equals(MEDIA_ATTRIBUTE)) {
                                String browser = st.nextToken();
                                agent = browser.substring(1, browser.length() - 1);
                            }
                        }
                        
                        if (url != null) {
                            if (agent != null) {
                                links.put(agent, url);
                            } else {
                                links.put(DEFAULT_BROWSER, url);
                            }
                        }
                    }
                }
            }            
        }
        return links;
    }
}